/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "environment.h"

// This constructor initializes all of the widgets and sets their properties. It
// then creates and shows the dialog. It receives the environment settings object
EnvironmentDialog::EnvironmentDialog (EnvironmentSettings *env) 
{
  fontname = Glib::ustring(env->get_font_face()) + " " + Glib::ustring(env->get_font_size());
  
  // Create objects for dialog
  initialize_objects();
  
  // Create the General editor tab
  tab_size->set_value(env->get_tabsize());
  asave_minutes->set_value(env->get_autosave());
  undo_actions->set_value(env->get_undo());
  replace_tabs->set_active(env->get_replace_tabs());
  load_on_launch->set_active(env->get_autoload());
  autoindent->set_active(env->get_auto_indent());
  
  // Create the View tab
  text_wrap->set_active(env->get_wrap());
  split_words->set_active(env->get_wrapsplit());
  bracket->set_active(env->get_brackets());
  right_margin->set_active(env->get_rightmargin());
  
  // Create the Fonts & Colors tab
  error->set_color(Gdk::Color(env->get_error_color()));
  warning->set_color(Gdk::Color(env->get_warning_color()));
  fontbutton->set_font_name(Glib::ustring(env->get_font_face() + " " + env->get_font_size()));
}

Gtk::Dialog* EnvironmentDialog::get_dialog()
{ return dialog;
}

// Initialize all of the objects and set all of the labels to be bold
void EnvironmentDialog::initialize_objects()
{
  Pango::FontDescription fd("Sans Bold 10");
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/environment.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("button_cancel", cancel);
  xml->get_widget("button_ok", ok);
  xml->get_widget("auto_save", asave_minutes);
  xml->get_widget("tab_width", tab_size);
  xml->get_widget("undo_actions", undo_actions);
  xml->get_widget("auto_indent", autoindent);
  xml->get_widget("highlight_brackets", bracket);
  xml->get_widget("load_on_launch", load_on_launch);
  xml->get_widget("insert_spaces", replace_tabs);
  xml->get_widget("split_words", split_words);
  xml->get_widget("text_wrapping", text_wrap);
  xml->get_widget("display_margin", right_margin);
  xml->get_widget("fontbutton", fontbutton);
  xml->get_widget("error_color", error);
  xml->get_widget("warning_color", warning);
  xml->get_widget("label_tabstops", label_tabstops);
  xml->get_widget("label_autoindent", label_autoindent);
  xml->get_widget("label_filemanagement", label_filemanagement);
  xml->get_widget("label_undo", label_undo);
  xml->get_widget("label_textwrapping", label_text_wrap);
  xml->get_widget("label_bracketmatch", label_bracket);
  xml->get_widget("label_rightmargin", label_rightmargin);
  xml->get_widget("label_font", label_font);
  xml->get_widget("label_compiler", label_compiler);
  
  // Set the labels to have a bold font
  label_autoindent->modify_font(fd);
  label_bracket->modify_font(fd);
  label_compiler->modify_font(fd);
  label_filemanagement->modify_font(fd);
  label_font->modify_font(fd);
  label_tabstops->modify_font(fd);
  label_text_wrap->modify_font(fd);
  label_undo->modify_font(fd);
  label_rightmargin->modify_font(fd);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
EnvironmentDialog::~EnvironmentDialog()
{
  delete cancel;
  delete ok;
  delete label_tabstops;
  delete label_filemanagement;
  delete label_font;
  delete label_compiler;
  delete label_autoindent;
  delete label_undo;
  delete tab_size;
  delete asave_minutes;
  delete undo_actions;
  delete replace_tabs;
  delete load_on_launch;
  delete autoindent;
  delete error;
  delete warning;
  delete fontbutton;
}

// Get the font size from the FontButton
Glib::ustring EnvironmentDialog::get_font_size()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  
  while (font.find(" ", 0) != string::npos)
    font.erase(0, font.find(" ", 0)+1);
  
  return font;
}

// Get the font face from the FontButton
Glib::ustring EnvironmentDialog::get_font_face()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  
  int loc = font.find(' ', 0);
  while (font.find(' ', loc+1) != string::npos)
    loc = font.find(' ', loc+1);
  
  font.erase(loc, font.length()-loc);
  return font;
}

// These functions allow the openldev class to get private members of this dialog
int EnvironmentDialog::get_asave_minutes()
{ return static_cast<int>(asave_minutes->get_value()); 
}

int EnvironmentDialog::get_undo()
{ return static_cast<int>(undo_actions->get_value()); 
}

int EnvironmentDialog::get_tab_size()
{ return static_cast<int>(tab_size->get_value()); 
}

bool EnvironmentDialog::get_load_on_launch()
{ return load_on_launch->get_active(); 
}

bool EnvironmentDialog::get_replace_tabs()
{ return replace_tabs->get_active(); 
}

bool EnvironmentDialog::get_auto_indent()
{ return autoindent->get_active(); 
}

bool EnvironmentDialog::get_wrap()
{ return text_wrap->get_active(); 
}

bool EnvironmentDialog::get_split()
{ return split_words->get_active(); 
}

bool EnvironmentDialog::get_brackets()
{ return bracket->get_active(); 
}

bool EnvironmentDialog::get_margin()
{ return right_margin->get_active(); 
}

Glib::ustring get_color_string(Gdk::Color color)
{
  char blue[3], green[3], red[3];
  snprintf(blue, sizeof(blue), "%X", color.get_blue());
  snprintf(red, sizeof(red), "%X", color.get_red());
  snprintf(green, sizeof(green), "%X", color.get_green());
  
  Glib::ustring clr = "#" + Glib::ustring(red);
  if (color.get_red() == 0)
    clr += "0";
  clr += green;
  if (color.get_green() == 0)
    clr += "0";
  clr += blue;
  if (color.get_blue() == 0)
    clr += "0";
  
  return clr;
}

Glib::ustring EnvironmentDialog::get_error_color()
{ return get_color_string(error->get_color()); 
}

Glib::ustring EnvironmentDialog::get_warning_color()
{ return get_color_string(warning->get_color());
}
