/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_properties.h"

// The project properties dialog initializes the object, set their properties,
// creates the dialog & shows it. It receives the project properties object.
ProjectPropertiesDialog::ProjectPropertiesDialog (ProjectSettings* project)
{
  // Create Makefile.am location string
  makefile_location = project->get_location() + "src/Makefile.am";
  initialize_objects();
  
  // Create entries for project properties
  name->set_text(project->get_name());
  author->set_text(project->get_author());
  email->set_text(project->get_email());
  homepage->set_text(project->get_url());
  version->set_text(project->get_version());
  
  // Place the add button & compiler commands entry in an hbox together
  add->signal_clicked().connect(sigc::mem_fun(*this, &ProjectPropertiesDialog::on_add));
  
  // Populate the command_list ListStore and load the commands from the Makefile.am
  populate_liststore();
  load_compiler_commands_from_makefile();
  dialog->set_default(*ok);
}

Gtk::Dialog* ProjectPropertiesDialog::get_dialog()
{ return dialog;
}

// Initialize all of the widgest for the project information dialog.
void ProjectPropertiesDialog::initialize_objects()
{
  Glib::ustring gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectproperties.glade";
  Glib::RefPtr<Gnome::Glade::Xml> xml = Gnome::Glade::Xml::create(gladefile);
  
  xml->get_widget("dialog", dialog);
  xml->get_widget("button_cancel", cancel);
  xml->get_widget("button_ok", ok);
  xml->get_widget("button_add", add);
  xml->get_widget("name", name);
  xml->get_widget("author", author);
  xml->get_widget("email", email);
  xml->get_widget("homepage", homepage);
  xml->get_widget("version", version);
  xml->get_widget("commands", compiler);
  xml->get_widget("treeview", command_list);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
ProjectPropertiesDialog::~ProjectPropertiesDialog()
{
  delete cancel;
  delete ok;
  delete add;
  delete name;
  delete author;
  delete email;
  delete homepage;
  delete version;
  delete compiler;
  delete command_list;
}

// Populate the command_list ListStore with all of the types of compiler
// commands that are available by default and their descriptions. The user
// can add their own by adding it directly to the Gtk::Entry
void ProjectPropertiesDialog::populate_liststore()
{  
  model = Gtk::ListStore::create(columns);
  
  // Add the rows to the ListStore
  add_row("-g", "Produce debugging information in native format");
  add_row("-Idir", "Add a directory to search for header files");
  add_row("-O0", "Do not optimize at all");
  add_row("-O1", "Reduce code size & execution time");
  add_row("-O2", "Optimize without sacrificing speed");
  add_row("-O3", "High level optimization");
  add_row("-Os", "Optimize only to reduce code size");
  add_row("-w", "Suppress all warnings");
  add_row("-Wall", "Turn on all standard warning messages");
  
  // Set the model to the ListStore & append the columns
  command_list->set_model(model);
  command_list->append_column("Command", columns.col_command);
  command_list->append_column("Description", columns.col_description);
}

// Add a row to the ListStore model. This function receives the command name
// and its description.
void ProjectPropertiesDialog::add_row(Glib::ustring comm, Glib::ustring descrip)
{
  Gtk::TreeModel::Row row = *(model->append());
  row[columns.col_command] = comm;
  row[columns.col_description] = descrip;
}

// Load the compiler commands from the Makefile.am
void ProjectPropertiesDialog::load_compiler_commands_from_makefile()
{
  Glib::ustring text;
  gint location;
  
  text = read_file(makefile_location);
  
  // Simplify to get out the compiler flags & remove returns and \'s
  location = text.find("FLAGS =", 0);
  text.erase(0, location+7);
  while (text.find("\n", 0) != string::npos)
    text.erase(text.find("\n", 0), 1);
  
  while (text.find("\\", 0) != string::npos)
    text.erase(text.find("\\", 0), 1);
  
  location = text.find("bin_PROGRAMS", 0);
  text.erase(location, text.length()-location);
  
  // Simplify white space
  location = text.find("-", 0);
  text.erase(0, location);
  while (text.find("  ", 0) != string::npos)
    text.erase(text.find("  ", 0), 1);
  
  while (text.find("\t", 0) != string::npos)
    text.erase(text.find("\t", 0), 1);
  
  compiler->set_text(text);
}

// Add the selected compiler command to the compiler entry. The function
// receives the command string. It makes sure the -Wall and -w commands
// are not shown at the same time and that -Idir is replaced by
// -I/usr/include/directory
void ProjectPropertiesDialog::add_compiler_command(Glib::ustring c)
{
  Glib::ustring text = compiler->get_text();
  int location = text.find(c, 0);
  
  // Check if it already exists
  if (location < 0)
  {
    // Add optimization command (Remove others)
    if (c.find("-O", 0) != string::npos)
    {
      if (text.find("-O", 0) != string::npos)
        text.erase(text.find("-O", 0), 4);
    }
      
    // Add the directory to the include command
    else if (c == "-Idir")
      c = "-I/usr/include/directory";
    
    // Make sure warning commands do not conflict
    else if (c == "-Wall" && text.find("-w", 0) != string::npos)
      text.erase(text.find("-w", 0), 3);
    else if (c == "-w" && text.find("-Wall", 0) != string::npos)
      text.erase(text.find("-Wall", 0), 6);
  
    text += " " + c;
    compiler->set_text(text);
  }
}

// Check which command is selected by getting it from the TreeView. Call the
// function to add it to the compiler commands.
void ProjectPropertiesDialog::on_add()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = command_list->get_selection();
  Gtk::TreeModel::iterator iter = selection->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring text;
    row->get_value(0, text);
    add_compiler_command(text);
  }
}

// Get private project properties and give the main openldev class access to them
Glib::ustring ProjectPropertiesDialog::get_name()
{ return name->get_text(); 
}

Glib::ustring ProjectPropertiesDialog::get_author()
{ return author->get_text(); 
}

Glib::ustring ProjectPropertiesDialog::get_email()
{ return email->get_text(); 
}

Glib::ustring ProjectPropertiesDialog::get_homepage()
{ return homepage->get_text(); 
}

Glib::ustring ProjectPropertiesDialog::get_version()
{ return version->get_text(); 
}

Glib::ustring ProjectPropertiesDialog::get_commands()
{ return compiler->get_text(); 
}
